/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.config.shared;

public abstract class AbstractCfgObject {
  private final SourceInfo source;

  protected AbstractCfgObject(SourceInfo source) {
    this.source = source;
  }

  public SourceInfo getSource() {
    return source;
  }

  public abstract int resolveInt();
  public abstract double resolveDouble();
  public abstract String resolveString();
  public abstract CfgObject resolveObject();
  public abstract CfgArray resolveArray();

  public abstract void accept(CfgVisitor visitor);

  public static class CfgVisitor {
    public void visit(AbstractCfgObject cfg) {

    }

    public void visitDouble(CfgDoubleLiteral cfg) {
      visit(cfg);
    }
    public void visitString(CfgStringLiteral cfg) {
      visit(cfg);
    }
    public void visitInt(CfgIntLiteral cfg) {
      visit(cfg);
    }

    public void visitObject(CfgObject cfg) {
      visit(cfg);
    }

    public void visitArray(CfgArray cfg) {
      visit(cfg);
    }

    public void visitReference(CfgReference cfg) {
      visit(cfg);
    }

    public void visitExpression(CfgExpression cfg) {
      visit(cfg);
    }

    public void visitFuncCall(CfgFuncCall cfg) {
      visit(cfg);
    }
  }

}
