/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.config.shared;

import java.util.Collections;
import java.util.List;

public class CfgFuncCall extends AbstractCfgObject {
  private final CfgObject parent;
  private final String method;
  private final List<AbstractCfgObject> args;

  public CfgFuncCall(SourceInfo source, CfgObject parent, String method, List<AbstractCfgObject> args) {
    super(source);
    this.parent = parent;
    this.method = method;
    this.args = args;
  }

  public String getMethod() {
    return method;
  }

  public List<AbstractCfgObject> getArgs() {
    return Collections.unmodifiableList(args);
  }

  @Override
  public int resolveInt() {
    return ThemerUtils.factory.invokeWith(getSource(), parent).intMethod(method, args);
  }

  @Override
  public double resolveDouble() {
    return ThemerUtils.factory.invokeWith(getSource(), parent).doubleMethod(method, args);
  }

  @Override
  public String resolveString() {
    return ThemerUtils.factory.invokeWith(getSource(), parent).stringMethod(method, args);
  }

  @Override
  public CfgObject resolveObject() {
    return ThemerUtils.factory.invokeWith(getSource(), parent).objectMethod(method, args);
  }

  @Override
  public CfgArray resolveArray() {
    return null; //TODO we dont have any array methods
  }

  @Override
  public void accept(CfgVisitor visitor) {
    visitor.visitFuncCall(this);
  }
}
