/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.slicer.client.job.grid;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.editor.client.Editor;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.data.shared.ModelKeyProvider;
import com.sencha.gxt.data.shared.PropertyAccess;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData.GradientDir;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData.Side;
import com.sencha.gxt.themebuilder.slicer.client.job.AbstractJob;
import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
import com.sencha.gxt.widget.core.client.grid.ColumnHeader;
import com.sencha.gxt.widget.core.client.grid.ColumnModel;
import com.sencha.gxt.widget.core.client.grid.Grid;

/**
 *
 */
public class GridColumnHeaderJob extends AbstractJob<Grid<?>> {
  public static final class SomeBean {
    private int id;
    private String column1;
    private String column2;

    public int getId() {
      return id;
    }

    public void setId(int id) {
      this.id = id;
    }

    public String getColumn1() {
      return column1;
    }

    public void setColumn1(String column1) {
      this.column1 = column1;
    }

    public String getColumn2() {
      return column2;
    }

    public void setColumn2(String column2) {
      this.column2 = column2;
    }
  }

  public interface SomeBeanProperties extends PropertyAccess<SomeBean> {
    @Editor.Path("id")
    ModelKeyProvider<SomeBean> key();

    ValueProvider<SomeBean, String> column1();

    ValueProvider<SomeBean, String> column2();
  }

  private SomeBeanProperties props = GWT.create(SomeBeanProperties.class);


  @Override
  protected Grid<?> createWidget() {

    ListStore<SomeBean> store = new ListStore<SomeBean>(props.key());

    List<ColumnConfig<SomeBean, ?>> columnConfigs = new ArrayList<ColumnConfig<SomeBean, ?>>();
    columnConfigs.add(new ColumnConfig<SomeBean, String>(props.column1(), 50, "header"));
    columnConfigs.add(new ColumnConfig<SomeBean, String>(props.column2(), 50, "header"));

    ColumnModel<SomeBean> cm = new ColumnModel<SomeBean>(columnConfigs);

    Grid<SomeBean> grid = new Grid<SomeBean>(store, cm);
    setSize(grid);

    return grid;
  }

  @Override
  public void collectWidgetData(JobCallback callback) {

    ColumnHeader<?>.Head column1 = widget.getView().getHeader().getHead(0);
    doEvent(column1, column1.getElement());

    WidgetData columnHeaderData = createWidgetData("column" + getHeaderBtnSuffix());
    setBorders(columnHeaderData, column1.getElement());
    setBox(column1, columnHeaderData.getBox());
    columnHeaderData.setGradient(GradientDir.bottom);
    columnHeaderData.setStretch(Side.BOTTOM);
    columnHeaderData.getSlices().setBg("grid/column-header" + getHeaderBtnSuffix() + ".png");
    callback.addWidgetData(columnHeaderData);


    ColumnHeader<?>.Head column2 = widget.getView().getHeader().getHead(1);
    //mouseover the column
    column2.onBrowserEvent(createMouseOverEvent(column2.getElement()));
    XElement a = column2.getElement().selectNode("a");//header button
    doEvent(column2, a);

    WidgetData columnBtnData = createWidgetData("column-header-btn-bg" + getHeaderBtnSuffix());
    setBorders(columnBtnData, a);
    setBox(a, columnBtnData.getBox());
    columnBtnData.setGradient(GradientDir.bottom);
    columnBtnData.setStretch(Side.BOTTOM);
    columnBtnData.getSlices().setBg("grid/column-header-btn-bg" + getHeaderBtnSuffix() + ".png");
    callback.addWidgetData(columnBtnData);
  }
  protected void doEvent(Widget w, XElement a) {

  }
  protected String getHeaderBtnSuffix() {
    return "";
  }
}
