/**
 * Sencha GXT 4.0.0 - Sencha for GWT
 * Copyright (c) 2006-2015, Sencha Inc.
 *
 * licensing@sencha.com
 * http://www.sencha.com/products/gxt/license/
 *
 * ================================================================================
 * Open Source License
 * ================================================================================
 * This version of Sencha GXT is licensed under the terms of the Open Source GPL v3
 * license. You may use this license only if you are prepared to distribute and
 * share the source code of your application under the GPL v3 license:
 * http://www.gnu.org/licenses/gpl.html
 *
 * If you are NOT prepared to distribute and share the source code of your
 * application under the GPL v3 license, other commercial and oem licenses
 * are available for an alternate download of Sencha GXT.
 *
 * Please see the Sencha GXT Licensing page at:
 * http://www.sencha.com/products/gxt/license/
 *
 * For clarification or additional options, please contact:
 * licensing@sencha.com
 * ================================================================================
 *
 *
 * ================================================================================
 * Disclaimer
 * ================================================================================
 * THIS SOFTWARE IS DISTRIBUTED "AS-IS" WITHOUT ANY WARRANTIES, CONDITIONS AND
 * REPRESENTATIONS WHETHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE
 * IMPLIED WARRANTIES AND CONDITIONS OF MERCHANTABILITY, MERCHANTABLE QUALITY,
 * FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, NON-INFRINGEMENT, PERFORMANCE AND
 * THOSE ARISING BY STATUTE OR FROM CUSTOM OR USAGE OF TRADE OR COURSE OF DEALING.
 * ================================================================================
 */
package com.sencha.gxt.themebuilder.slicer.client.job.tabs;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.Label;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.themebuilder.slicer.client.data.BoxData;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData;
import com.sencha.gxt.themebuilder.slicer.client.data.WidgetData.Side;
import com.sencha.gxt.themebuilder.slicer.client.job.AbstractJob;
import com.sencha.gxt.widget.core.client.PlainTabPanel;
import com.sencha.gxt.widget.core.client.PlainTabPanel.PlainTabPanelAppearance;
import com.sencha.gxt.widget.core.client.TabItemConfig;

/**
 *
 */
public class TabPanelJob extends AbstractJob<PlainTabPanel> {
  private static final String TAB_TEXT = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";

  public PlainTabPanelAppearance getAppearance() {
    return GWT.create(PlainTabPanelAppearance.class);
  }

  @Override
  protected PlainTabPanel createWidget() {
    PlainTabPanel panel = new PlainTabPanel(getAppearance());

    panel.setPixelSize(250, 70);
    TabItemConfig tab1Config = new TabItemConfig();
    tab1Config.setHTML(TAB_TEXT);
    panel.add(new Label("Tab 1 content: " + getAppearance().getClass()), tab1Config);

    TabItemConfig tab2Config = new TabItemConfig();
    tab2Config.setHTML(TAB_TEXT);
    panel.add(new Label("Tab 2 content"), tab2Config);

    TabItemConfig tab3Config = new TabItemConfig();
    tab3Config.setHTML(TAB_TEXT);
    panel.add(new Label("Tab 3 content"), tab3Config);
    getAppearance().onMouseOver(panel.getElement(), (XElement) getAppearance().getBar(panel.getElement()).select(getAppearance().getItemSelector()).getItem(2));

    return panel;
  }

  private native Event fakeEventWithTarget(Element item) /*-{
    return {target:item};
  }-*/;

  @Override
  public void collectWidgetData(JobCallback callback) {
    XElement bar = getAppearance().getBar(widget.getElement());

    NodeList<Element> items = bar.select(getAppearance().getItemSelector());


    WidgetData tab1Data = createWidgetData("tab");
    XElement tab1 = items.getItem(0).cast();
    setBorders(tab1Data, tab1);
    setRadius(tab1Data, tab1);
    setBox(tab1, tab1Data.getBox());
    tab1Data.setStretch(stretchSide());

    tab1Data.getSlices().setBg("tabs/" + imageFileSuffix() + "-bg.png");

    callback.addWidgetData(tab1Data);



    WidgetData tabSides = createWidgetData("tab-sides");

    setRadius(tabSides, 24);
    setBorders(tabSides, tab1);

    setBox(tab1, tabSides.getBox());
    BoxData tabBox = tabSides.getBox();
    // make sure corner is included in this slice
    tabBox.setY(tabBox.getY() - 24);
    tabBox.setH(tabBox.getH() + 48);
    tabSides.setStretch(stretchSide());
    tabSides.getSlices().setSides("tabs/" + imageFileSuffix() + ".png");

    callback.addWidgetData(tabSides);


    WidgetData inactiveTab = createWidgetData("inactive");
    XElement tab2 = items.getItem(1).cast();
    setBorders(inactiveTab, tab2);
    setRadius(inactiveTab, tab2);
    setBox(tab2, inactiveTab.getBox());
    inactiveTab.setStretch(stretchSide());

    inactiveTab.getSlices().setBg("tabs/inactive-" + imageFileSuffix() + "-bg.png");

    callback.addWidgetData(inactiveTab);


    WidgetData inactiveTabSides = createWidgetData("inactive-sides");

    setRadius(inactiveTabSides, 24);
    setBorders(inactiveTabSides, tab2);

    setBox(tab2, inactiveTabSides.getBox());
    BoxData inactiveSideBox = inactiveTabSides.getBox();
    // make sure corner is included in this slice
    inactiveSideBox.setY(inactiveSideBox.getY() - 24);
    inactiveSideBox.setH(inactiveSideBox.getH() + 48);
    inactiveTabSides.setStretch(stretchSide());
    inactiveTabSides.getSlices().setSides("tabs/inactive-" + imageFileSuffix() + ".png");

    callback.addWidgetData(inactiveTabSides);


    WidgetData hoverTab = createWidgetData("hover");
    XElement tab3 = items.getItem(2).cast();
    setBorders(hoverTab, tab3);
    setRadius(hoverTab, tab3);
    setBox(tab3, hoverTab.getBox());
    hoverTab.setStretch(stretchSide());

    hoverTab.getSlices().setBg("tabs/hover-" + imageFileSuffix() + "-bg.png");

    callback.addWidgetData(hoverTab);


    WidgetData hoverTabSides = createWidgetData("hover-sides");

    setRadius(hoverTabSides, 24);
    setBorders(hoverTabSides, tab3);

    setBox(tab3, hoverTabSides.getBox());
    BoxData hoverSideBox = hoverTabSides.getBox();
    // make sure corner is included in this slice
    hoverSideBox.setY(hoverSideBox.getY() - 24);
    hoverSideBox.setH(hoverSideBox.getH() + 48);
    hoverTabSides.setStretch(stretchSide());
    hoverTabSides.getSlices().setSides("tabs/hover-" + imageFileSuffix() + ".png");

    callback.addWidgetData(hoverTabSides);


  }

  protected Side stretchSide() {
    return Side.BOTTOM;
  }

  protected String imageFileSuffix() {
    return "tab";
  }
}
